#include <nds.h>
#include <stdlib.h>
#include "smi_startup_viaipc.h"
#include "MessageQueue.h"
#include "wifi_hal.h"

//---------------------------------------------------------------------------------
#define allowed_to_sleep (*(volatile int*)0x027FDF00)
void enter_sleep_mode()
{
	if(!allowed_to_sleep)
		return;

	u32 oldIE = REG_IE; // Save the current interrupt sate.
	swiChangeSoundBias(0, 0x400); // Turn the speaker down.

	int power = readPowerManagement(PM_CONTROL_REG); // Save current power state.
	writePowerManagement(PM_CONTROL_REG, PM_LED_CONTROL(1)); // Set sleep LED.

	REG_IE = IRQ_LID; // Register for the lid interrupt, and only that IRQ.

	// Power down till we get our interrupt.
	swiSleep();

	// Restore the interrupt state.
	REG_IF = ~0;
	REG_IE = oldIE;

	writePowerManagement(PM_CONTROL_REG, power); // Restore power state.

	swiChangeSoundBias(1, 0x400); // Turn the speaker up.
}

//---------------------------------------------------------------------------------
void startSound(int sampleRate, const void* data, u32 bytes, u8 channel, u8 vol,  u8 pan, u8 format) {
//---------------------------------------------------------------------------------
	SCHANNEL_TIMER(channel)  = SOUND_FREQ(sampleRate);
	SCHANNEL_SOURCE(channel) = (u32)data;
	SCHANNEL_LENGTH(channel) = bytes >> 2 ;
	SCHANNEL_CR(channel)     = SCHANNEL_ENABLE | SOUND_ONE_SHOT | SOUND_VOL(vol) | SOUND_PAN(pan) | (format==1?SOUND_8BIT:SOUND_16BIT);
}

//---------------------------------------------------------------------------------
s32 getFreeSoundChannel() {
//---------------------------------------------------------------------------------
	int i;
	for (i=0; i<16; i++) {
		if ( (SCHANNEL_CR(i) & SCHANNEL_ENABLE) == 0 ) return i;
	}
	return -1;
}

touchPosition first,tempPos;

//---------------------------------------------------------------------------------
void VcountHandler() {
//---------------------------------------------------------------------------------
	static int lastbut = -1;
	
	uint16 but=0, x=0, y=0, xpx=0, ypx=0, z1=0, z2=0;

	but = REG_KEYXY;

	if (!( (but ^ lastbut) & (1<<6))) {
 
		tempPos = touchReadXY();

		if ( tempPos.x == 0 || tempPos.y == 0 ) {
			but |= (1 <<6);
			lastbut = but;
		} else {
			x = tempPos.x;
			y = tempPos.y;
			xpx = tempPos.px;
			ypx = tempPos.py;
			z1 = tempPos.z1;
			z2 = tempPos.z2;
		}
		
	} else {
		lastbut = but;
		but |= (1 <<6);
	}

	IPC->touchX			= x;
	IPC->touchY			= y;
	IPC->touchXpx		= xpx;
	IPC->touchYpx		= ypx;
	IPC->touchZ1		= z1;
	IPC->touchZ2		= z2;
	IPC->buttons		= but;

	// Check if the lid has been closed.
	if(but & BIT(7)) {
		enter_sleep_mode();
	}
}

//---------------------------------------------------------------------------------
void VblankHandler(void) {
//---------------------------------------------------------------------------------

	u32 i;

	//sound code  :)
	TransferSound *snd = IPC->soundData;
	IPC->soundData = 0;

	if (0 != snd) {

		for (i=0; i<snd->count; i++) {
			s32 chan = getFreeSoundChannel();

			if (chan >= 0) {
				startSound(snd->data[i].rate, snd->data[i].data, snd->data[i].len, chan, snd->data[i].vol, snd->data[i].pan, snd->data[i].format);
			}
		}
	}
}

//---------------------------------------------------------------------------------
int main(int argc, char ** argv) {
//---------------------------------------------------------------------------------
	readUserSettings();

	//enable sound
	powerON(POWER_SOUND);
	writePowerManagement(PM_CONTROL_REG, ( readPowerManagement(PM_CONTROL_REG) & ~PM_SOUND_MUTE ) | PM_SOUND_AMP );
	SOUND_CR = SOUND_ENABLE | SOUND_VOL(0x7F);
	IPC->soundData = 0;
	allowed_to_sleep = 1;

	irqInit();
	initClockIRQ();
	SetYtrigger(80);
	irqSet(IRQ_VBLANK, VblankHandler);
	irqSet(IRQ_VCOUNT, VcountHandler);
	irqEnable(IRQ_VBLANK) ;
	irqEnable(IRQ_VCOUNT);

	IPC_Init() ;
	LWIFI_Init() ;
	
	while (1) {
		LWIFI_IPC_UpdateNOIRQ() ;
	} ;	
}

